local addonName, addon = ...

local actionName, actionTag = { }, addon.actionTag

local format, next, strsplit, tonumber = string.format, next, strsplit, tonumber

local GetItemNameByID, IsItemDataCachedByID = C_Item.GetItemNameByID, C_Item.IsItemDataCachedByID
local DoesSpellExist, IsSpellDataCached = C_Spell.DoesSpellExist, C_Spell.IsSpellDataCached
local GetSpellInfo = GetSpellInfo

local AddActionName, RequestItemData, RequestSpellData

do
	local header, Execute, SafeCall = addon.header, addon.header.Execute, addon.SafeCall

	function AddActionName(actionID, name)
		actionName[actionID] = name
		SafeCall(Execute, header, format("actionName[%q] = %q", actionID, name))
	end
end

do
	local queue, registered, RequestLoadItemDataByID = { }, nil, C_Item.RequestLoadItemDataByID

	local function OnItemDataLoad(self, event, itemID, success)
		local actionID = queue[itemID]
		if actionID then
			queue[itemID] = nil
			if not next(queue) then
				registered = nil
				addon:UnregisterEvent(event)
			end
			if success then
				local name = GetItemNameByID(itemID)
				if name then
					AddActionName(actionID, name)
				end
			end
		end
	end

	function RequestItemData(actionID, itemID)
		if not queue[itemID] then
			queue[itemID] = actionID
			if not registered then
				registered = true
				addon:RegisterEvent('ITEM_DATA_LOAD_RESULT', OnItemDataLoad)
			end
			RequestLoadItemDataByID(itemID)
		end
	end
end

do
	local queue, registered, RequestLoadSpellData = { }, nil, C_Spell.RequestLoadSpellData

	local function OnSpellDataLoad(self, event, spellID, success)
		local actionID = queue[spellID]
		if actionID then
			queue[spellID] = nil
			if not next(queue) then
				registered = nil
				addon:UnregisterEvent(event)
			end
			if success then
				local name = GetSpellInfo(spellID)
				if name then
					AddActionName(actionID, name)
				end
			end
		end
	end

	function RequestSpellData(actionID, spellID)
		if not queue[spellID] then
			queue[spellID] = actionID
			if not registered then
				registered = true
				addon:RegisterEvent('SPELL_DATA_LOAD_RESULT', OnSpellDataLoad)
			end
			RequestLoadSpellData(spellID)
		end
	end
end

addon.actionName = setmetatable(actionName, { __index = function(self, actionID)
	local actionType, typeID = strsplit(":", actionID)
	if actionType and typeID then
		typeID = tonumber(typeID)
		local name
		if actionType == 'spell' then
			if DoesSpellExist(typeID) then
				if IsSpellDataCached(typeID) then
					name = GetSpellInfo(typeID)
				else
					RequestSpellData(actionID, typeID)
				end
			end
		elseif actionType == 'item' then
			if IsItemDataCachedByID(typeID) then
				name = GetItemNameByID(typeID)
			else
				RequestItemData(actionID, typeID)
			end
		end
		if name then
			AddActionName(actionID, name)
			return name
		end
	end
end })

--[[----------------------------------------------------------------------------
Initialize
GetSpellBookItemInfo - Returns a spellID not modified by talents
GetSpellBookItemName - Returns a spellID modified by talents

local overrideSpellID = FindSpellOverrideByID(spellID)
local baseSpellID     = FindBaseSpellByID(spellID)
------------------------------------------------------------------------------]]
addon:RegisterEvent('PLAYER_LOGIN', function(self, event)
	self:UnregisterEvent(event)

	local config = addon.config
	addon.config = nil

	for actionID in pairs(addon.actionMode) do
		if actionName[actionID] then
			-- Force __index hit
		end
	end

	-- ToDo: Add all specs if none specified and has tag playerClass
	local numSpecs, specTag = GetNumSpecializations(), { }
	for index = 1, numSpecs do
		local tag = format("%s:%s", 'spec', index)
		specTag[index], specTag[tostring(index)], specTag[tag] = tag, tag, index
	end
	for actionID in pairs(actionTag[addon.playerClass]) do
		local addAllSpecs = true
		for index = 1, numSpecs do
			if actionTag[specTag[index]][actionID] then
				addAllSpecs = nil
				break
			end
		end
		if addAllSpecs then
			for index = 1, numSpecs do
				actionTag[specTag[index]][actionID] = specTag[index]
			end
		end
	end

	addon.config = config
end)
